/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "alignment.h"

Alignment::Alignment()
{
// 	seqNames = StringVec();
// 	sequences = StringVec();
	ntaxa = 0;
	fullLength = 0;
}

Alignment::Alignment(const char *inFile)
{
	readAlign(inFile);
}
	
void Alignment::readAlign (const char *alignFile)
{
	cout << "\nReading alignment file " << alignFile << "..." <<endl;
	ifstream in;
	try{
		in.exceptions (ios::failbit | ios::badbit);
		in.open(alignFile);
		readAlign(in);
		in.clear();
		in.exceptions (ios::failbit | ios::badbit);
		in.close();
	} catch (ios::failure){
		outError(ERR_READ_INPUT, alignFile);
	}
	cout << "Finish reading alignment file!" << endl;
	cout << "Alignment contains " << ntaxa << " sequences of length: " << fullLength << endl;
}

void Alignment::readAlign (ifstream &in)
{
	int seqNum;
	StringVec _seqNames, _sequences;
	int seqLen;
	string currentName, seqNumStr, seqLenStr, currentSeq;
	try {
		//get number of sequence in string
		in >> seqNumStr;
		//get sequence's length in string
		in >> seqLenStr;
		// convert to int
		seqNum = convert_int(seqNumStr.c_str());
		seqLen = convert_int(seqLenStr.c_str());		

		/*for (int i = 0; i < seqNum; i++)
		{
			//get the name of sequence and the sequence itself
			if ( !(in >> currentName >> currentSeq) ) break;
			_seqNames.push_back(currentName);
			//if sequence length is not valid, output an error			
			if (seqLen != currentSeq.size())
			{
				outError ("Error in sequence ", currentName);
			}
			_sequences.push_back(currentSeq);
		}*/		
		for (int i = 0; i < seqNum; i++)
		{
			//get the name of sequence
			if ( !(in >> currentName) ) break;
			_seqNames.push_back(currentName);
			
			if (!getline(in,currentSeq,'\n')) break;			
			removeSpaces(currentSeq);
			_sequences.push_back(currentSeq);
		}	
		int index = 0;
		bool stop = (_sequences[0].size() == seqLen);
		while(!in.eof() & !stop)
		{
			in.ignore(5000,'\n');
// 			cout << "in while: " << index << endl;
			index++;	
			for (int i = 0; i < seqNum; i++)
			{
				if (!getline(in,currentSeq,'\n')) 
				{
					break;								
					stop = true;
				}
				removeSpaces(currentSeq);
				_sequences[i] += currentSeq;
// 				cout << "\t" << currentSeq << endl;
			}
			if (_sequences[0].size() == seqLen)
				stop = true;		
		}
	} catch (bad_alloc) {
			outError(ERR_NO_MEMORY);
	} catch (const char* str) {
			outError(str);
	} catch (char *str) {
			outError(str);
	} catch (string str) {
			outError(str);
	} catch (ios::failure) {
			outError(ERR_READ_INPUT);
	} catch (...) {
			outError(ERR_READ_ANY);
	}
	
	//test validity
	for (int i = 0; i < seqNum; i++)
	{
		if (seqLen != _sequences[i].size())
		{
			outError ("Error in sequence ", _seqNames[i]);
		}
	}

	//return
	ntaxa = seqNum;
	fullLength = seqLen;
	seqNames = _seqNames;
	sequences = _sequences;
}

Alignment::~Alignment()
{
	/*seqNames.clear();
	sequences.clear();*/
}


