/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ALIGNMENT_H
#define ALIGNMENT_H

#include "utility.h"

/**
Describe an alignment with string vector of sequence names and string vector of sequences.

	@author Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler <{minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at>
*/
class Alignment{
/** ========== methods ====================*/
public:
	/**
		empty constructor
	*/
    Alignment();

	/**
		constructing from an alignment in sequential PHYLIP format
		@param inFile file contains the alignment in sequential PHYLIP format
	*/
	Alignment(const char *inFile);

	/**
		read the alignment from an alignment file
		@param inFile file contains the alignment in sequential PHYLIP format
	*/
	void readAlign (const char *inFile);
	
	/**
		read the alignment from an input stream
		@param in the input stream containing alignment in sequentail PHYLIP format
	*/
	void readAlign (ifstream &in);

	/**
		destructor
	*/
    ~Alignment();
/** ================ atributes =========================*/
public:
	/**
		vector of sequences' names
	*/
	StringVec seqNames;
	/**
		string vector contains the sequences
	*/
	StringVec sequences;
	/**
		number of sequences in the alignment
	*/
	int ntaxa;
	/**
		length of the alignment
	*/
	int fullLength;
};

#endif
