/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BIPARTITEMINCOST_H
#define BIPARTITEMINCOST_H

#include "utility.h"

#define maxC 1000000000

/**
To compute MinCostFlow on Bipartite graph (solving the pattern matching problem)

	@author Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler <{minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at>
*/
class BipartiteMinCost{
public:
	//default constructor
	BipartiteMinCost(){};
	/**
		constructor from main components
		@param _a vector contains amount of each pattern at each node on the left hand side
		@param _b vector contains amount of each patterns at each node on the right hand side
		@param _c matrix contains the matching cost		
	*/
    BipartiteMinCost(IntVec _a, IntVec _b, IntMatrix _c);

	/**
		constructor from input file
		@param inFile input file containing main components
		FORMAT of inFile: 
			<first line> m #number of nodes on the left hand side
			<second line> n #number of nodes on the right hand side
			<m next lines> each line contains the amount of each pattern (integer) on the left hand side
			<n next lines> each line contains the amount of each pattern (integer) on the right hand side
			<m*n matrix> the matching cost matrix
	*/
	BipartiteMinCost(const char* inFile);

	/**
		Destructor
	*/
    ~BipartiteMinCost();

	/**
		first initialization: get input, initialize some variables
	*/
	void firstInit();
	/**
		solve the problem
	*/
	void solve();
    void init();
	void initBFS();
    void enlarge();
    void subX_addY();
    void findAugmentingPath();

//Attribute
public:
	/**
		Number of nodes (patterns) on the left hand side of the bipartite graph
	*/
	int m;
	
	/**
		Number of nodes (patterns) on the right hand side of the bipartite graph
	*/
	int n;
	/**
		Amount of the pattern at each node on the left hand side of the bipartite graph (observed more)
	*/
	IntVec a;

	/**
		Number of the pattern at each node on the left right side of the bipartite graph (observed more)
	*/
	IntVec b;

	/**
		Matching cost (mutation cost to transverse each pattern on the left hand side to each pattern on the right hand side)
	*/
	IntMatrix c;

	/**
		result: matching score
	*/
	int totalCost;
	
	/**
		result: the matching itself
	*/
	IntMatrix d;

//private data, used for calculating only
private:
	int maxN;
	IntVec dx, dy, delta;
	IntVec fx, fy;
	IntVec traceX, traceY, arg;
	IntVec deltaX, deltaY, queue;
	int start, finish, front, rear;
};

#endif

