/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "intervalset.h"

IntervalSet::IntervalSet()
{
}
IntervalSet::IntervalSet (const IntSet intSet)
{
	init(intSet);
}
IntervalSet::IntervalSet (const IntVec intVec)
{
	IntSet intSet;
	std::copy(intVec.begin(), intVec.end(), inserter(intSet, intSet.end()));
	init(intSet);	
}

void IntervalSet::init(const IntSet intSet)
{
	IntMatrix ret;
	int prev = -2;
	int index = -1;
	int current;
	for ( IntSet::const_iterator it = intSet.begin(); it != intSet.end(); it++)
	{
		current = (*it);
		if (current - prev > 1)
		{
			index++;
			IntVec temp;
			temp.push_back(current);
			temp.push_back(1);
			ret.push_back(temp);
		}
		else
		{
			ret[index][1]++;
		}
		prev = current;
	}
	int size = ret.size();
	for (int i = 0; i < size; i++)
	{
		int end = ret[i][0] + ret[i][1] - 1;
		IntInterval *curInter = new IntInterval(ret[i][0], end);
		(*this).insert(curInter);
	}
}

IntInterval* IntervalSet::findEnd(const int aInt)
{
	for (const_iterator it = begin(); it != end(); it++)
	{
		if (aInt == (*it)->getEnd())
			return (*it);
	}
	return NULL;
}

IntInterval* IntervalSet::findBegin(const int aInt)
{
	IntInterval *query = new IntInterval(aInt);
	iterator it = (*this).find(query);
	if (it != end())
		return (*it);
	return NULL;
}

void IntervalSet::addAnInterval (IntInterval *anInterval, int &amount, bool atEnd, bool added)
{
	//Note1: Added on 2009.12.01. check if this intervalset is empty
	if (empty())
	{
		if (anInterval->getAmount() > amount)
		{	
			//added on 2009.11.17
			IntInterval *toadd;			
			toadd = new IntInterval (anInterval->getBegin(), anInterval->getBegin()+amount-1);			
			amount = 0;
			insert(toadd);
		}
		else
		{
			amount -= anInterval->getAmount();
			insert(anInterval);				
		}
		anInterval->mark(true);
		return;
	} // End of note 1

	//search for the begin
	int curEnd = anInterval->getEnd() + 1;
	IntInterval *query = new IntInterval(curEnd);
	iterator it = (*this).find(query);	
	if (it != end())
	{
		if (amount < anInterval->getAmount())
		{
			int newBegin = curEnd - amount;
			(*it)->setBegin(newBegin);
			(*it)->updateAmount();
			amount = 0;
		}
		else
		{
			(*it)->setBegin(anInterval->getBegin());
			(*it)->updateAmount();
			amount -= anInterval->getAmount();
			
		}
		anInterval->mark(true);
		//traverse the updated list to see whether the newly added interval connects with its neighbors.
		if (it != begin())
		{
			iterator prev = it;
			prev--;
			if ((*prev)->getEnd() == (*it)->getBegin() - 1 )
			{
				(*it)->setBegin((*prev)->getBegin());
				(*it)->updateAmount();
				erase(prev);
			}
		}
	}
	else //not found a match, search for the end	
	{
		int curBegin = anInterval->getBegin() - 1;
		iterator findEnd;
		bool found = false;
		for (findEnd = begin(); findEnd != end() && !found; findEnd++)
		{
			if ((*findEnd)->getEnd() == curBegin) //found a match
			{
				if (amount < anInterval->getAmount())
				{
					int newEnd = curBegin + amount;
					(*findEnd)->setEnd(newEnd);
					(*findEnd)->updateAmount();
					amount = 0;
				}
				else
				{
					(*findEnd)->setEnd(anInterval->getEnd());
					(*findEnd)->updateAmount();
					amount -= anInterval->getAmount();					
				}
				//traverse the updated list to see whether the newly added interval connects with its neighbors.
				iterator next = findEnd;
				next++;
				if (next != end())
				{				
					if ((*next)->getBegin() == (*findEnd)->getEnd() +1 )
					{
						(*findEnd)->setEnd((*next)->getEnd());
						(*findEnd)->updateAmount();
						erase(next);
					}
				}
				anInterval->mark(true);
				//stop searching
				found = true;
				break;
			}			
		}
		if(!found && added)
		{
			if (anInterval->getAmount() > amount)
			{	
				//added on 2009.11.17
				IntInterval *toadd;
				if (!atEnd)
					toadd = new IntInterval (anInterval->getBegin(), anInterval->getBegin()+amount-1);
				else
					toadd = new IntInterval (anInterval->getEnd()-amount+1, anInterval->getEnd());
 				//IntInterval *toadd = new IntInterval (anInterval->getBegin(), anInterval->getBegin()+amount-1);
				amount = 0;
				insert(toadd);
			}
			else
			{
				amount -= anInterval->getAmount();
				insert(anInterval);				
			}
			anInterval->mark(true);
		}
	}	
	
}

void IntervalSet::addIntervalVec (IntervalVec &interVec, int &amount, bool added)
{
	int size = interVec.size();
	int totalAmount = 0;
	int stepAmount = amount;
	
	if (size > 0)
	{	
		for (int i = 0; i < size; i++)
		{
			totalAmount += interVec[i]->getAmount();			
		}
		assert(totalAmount >= amount);
		bool finish = false;
		for (int i = 0; i <size && !finish; i++)
		{			
			if (!interVec[i]->isMarked())
				addAnInterval(interVec[i], stepAmount, false);
			if (stepAmount == 0)
			{
				finish = true;
				break;
			}
		}
		/*if (!finish && added) // NEED CHANGE HERE
		{
			for (int i = 0; i <size && !finish; i++)
			{	
							
				if (!interVec[i]->isMarked())
				{
					addAnInterval(interVec[i], stepAmount, added);				
					if (stepAmount == 0)
					{
						finish = true;
						break;
					}
				}
			}
		}*/		
		//added on 2009.11.06
		if (!finish && added)
		{			
			while (!finish)
			{
				int index = -1;
				int lessDistance=1000000;
				int tempDistance;
				bool pos;
				//search for an interval from interVec with the less distance to this set, and pos contains indicates its begin or its end having the less distance
				for (int i = 0; i < size; i++)
				{	
					bool tempPos;			
					if (!interVec[i]->isMarked())
					{						
						tempDistance = lessDistanceTo(interVec[i],tempPos);						
						if (index == -1)
						{
							lessDistance = tempDistance;
							index = i;
							pos = tempPos;
						}
						else
							if (lessDistance > tempDistance)
							{
								lessDistance = tempDistance;
								index = i;
								pos = tempPos;
							}
					}					
				}
				if (index == -1)
					outError("Internal Error in IntervalSet::addIntervalVec: index==-1. Pls. inform the authors!");
				//add the interval with the less distance first
				addAnInterval(interVec[index],stepAmount,pos,added);
				if( stepAmount == 0)
				{
					finish = true;
					break;
				}
			}
		}
		//re-assign amount
		amount = stepAmount;
	}	
}

void IntervalSet::addIntVec (const IntVec intVec, int &amount, bool added)
{
	IntervalSet interSet(intVec);
	IntervalVec interVec(interSet.size());
	std::copy(interSet.begin(), interSet.end(), interVec.begin());
	std::sort(interVec.begin(), interVec.end(), amountCmp);
	addIntervalVec(interVec,amount, added);	
}

int IntervalSet::lessDistanceTo(const IntInterval *anInterval, bool &pos)
{
	if (empty())
	{
		pos =  true;
		return 0;
	}
	const_iterator it1 = begin();
	//anInterval lies at the begining of the current set
	if (anInterval->getEnd() <= (*it1)->getBegin())
	{
		pos = true;
		return ((*it1)->getBegin() - anInterval -> getEnd());
	}	
	const_iterator secondIt = it1;
	secondIt++;
	int retDistance = -1;
	//check if anInterval lies somewhere in the middle of the current set
	for (const_iterator it2 = secondIt; it2 != end(); it2++, it1++)
	{		
		int dis1 = anInterval->getBegin() - (*it1)->getEnd();
		int dis2 = (*it2)->getBegin() - anInterval->getEnd();		
		if ( dis1 >= 0 && dis2 >= 0)
		{
			retDistance = (dis1 < dis2) ? dis1:dis2;
			pos = (dis1 < dis2) ? false:true;		
		}	
	}
	//the interval lies in the middle of the current set
	if (retDistance != -1)
		return retDistance;
	//the interval should lie at the end of the current set.
	//If not, the input interval overlaps somewhere but it must not happen.		
	if (anInterval->getBegin() < (*it1)->getEnd())
		outError("internal error: the int interval (to check) overlaps with the current intervalSet. Please report to the authors!");
	else
	{
		pos = false;
		return (anInterval->getBegin() - (*it1)->getEnd());
	}
}

void IntervalSet::printIntervalSet(ostream &out) const
{
	out << "Start\tLength" << endl;
	for (const_iterator it = begin(); it != end(); it++)
	{
		/*if ((*it)->getAmount() >= 2)
			out << (*it)->getBegin() << "-" << (*it)->getEnd() << endl;
		else
			out << (*it)->getBegin() << endl;*/
		out << (*it)->getBegin() << "\t" << (*it)->getAmount() << endl;
	}
}

/*void IntervalSet::printIntervalSetFull(ostream &out) const
{
	for (const_iterator it = begin(); it != end(); it++)
	{
		if ((*it)->getAmount() >= 2)
		{
			for (int i = (*it)->getBegin(); i <= (*it)->getEnd(); i++)
			out << i << endl;
		}
		else
			out << (*it)->getBegin() << endl;
	}
}*/

IntervalSet::~IntervalSet()
{
}


