/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef INTINTERVAL_H
#define INTINTERVAL_H

#include "intinterval.h"
#include "utility.h"

/**
Store information about an interval of consecutive integers.

	@author Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler <{minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at>
*/
class IntInterval{
friend class PatternSet;
public:
	/**
		empty constructor
	*/
    IntInterval();
	/**
		constructor with 1 number
		@param abegin the only element of this interval
		begin = end = abegin, amount = 1
	*/
	IntInterval(const int abegin, const bool amarked = false);
	/**
		constructor with enough parameters
		@param abegin the begining of this interval
		@param aend the end of this interval
	*/
	IntInterval (const int abegin, const int aend, const bool amarked = false);
	/**
		get the begining
	*/
	inline int getBegin() const
	{
		return begin;
	}
	/**
		get the end
	*/
	inline int getEnd() const
	{
		return end;
	}
	/**
		get the amount
	*/
	inline int getAmount() const
	{
		return amount;
	}

	/**
		check whether this interval is marked
	*/
	inline bool isMarked() const
	{
		return marked;
	}
	/**
		(re) set the begining
		@param abegin the begining
	*/
	void setBegin(const int aInt);
	/**
		(re) set the end
		@param aend the end
	*/
	void setEnd(const int aInt);
	/**
		mark or unmark this interval
	*/
	void mark(const bool mark);
	/**
		update the length of this interval (after begin or end is modified)
	*/
	void updateAmount();	
    ~IntInterval();
public:
	/**
		the begining of this interval
	*/
	int begin;
	/**
		the end of this interval
	*/
	int end;
	/**
		the amount of numbers in this interval
	*/
	int amount;
	/**
		to mark (unmark) this interval
	*/
	bool marked;
};

/**
	to sort a vector of interval in descending order of the amount
*/
inline bool amountCmp(const IntInterval *a, const IntInterval *b)
{
	return (a->getAmount() > b->getAmount());
}

/**
	define a vector containing some intervals
*/
typedef vector <IntInterval* > IntervalVec;
/**
	define a 2*2 matrix of intervals
*/
typedef vector < vector<IntInterval* > > IntervalVecVec;

#endif
