/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "node.h"

Node::Node()
{
	id = -1;
	seqID = -1;
	fitchScore = 0;
	fitchSet = 0;
	finalFitchSet = 0;
	minExBrCost = IntVec(StateNum, MaxExBrCost);
	maxExBrCost = IntVec(StateNum, -1);
	charSet = IntVec(StateNum, 0);
	allCharSet = 0;
}

Node::Node(int aid)
{
	id = aid;
	seqID = -1;
	fitchScore = 0;
	fitchSet = 0;
	finalFitchSet = 0;
	minExBrCost = IntVec(StateNum, MaxExBrCost);
	maxExBrCost = IntVec(StateNum, -1);
	charSet = IntVec(StateNum, 0);
	allCharSet = 0;
}

Node::Node(int aid, int aname) {
	id = aid;
	name = "";
	name += aname;
	seqID = -1;
	fitchScore = 0;
	fitchSet = 0;
	finalFitchSet = 0;
	minExBrCost = IntVec(StateNum, MaxExBrCost);
	maxExBrCost = IntVec(StateNum, -1);
	charSet = IntVec(StateNum, 0);
	allCharSet = 0;
}

Node::Node(int aid, const char *aname) {
	id = aid;
	name = aname;
	seqID = -1;
	fitchScore = 0;
	fitchSet = 0;
	finalFitchSet = 0;
	minExBrCost = IntVec(StateNum, MaxExBrCost);
	maxExBrCost = IntVec(StateNum, -1);
	charSet = IntVec(StateNum, 0);
	allCharSet = 0;
}

bool Node::isLeaf()
{
	return neighbors.size() <= 1;
}

Node::~Node()
{
	NeighborVec::reverse_iterator it;
	for (it = neighbors.rbegin(); it != neighbors.rend(); it++)
		delete (*it);
	neighbors.clear();
	minExBrCost.clear();
	maxExBrCost.clear();
	charSet.clear();
}

void Node::replaceNeighbor(Node* node, Node *newnode, double newlen)
{
	for (NeighborVec::iterator it = neighbors.begin(); it != neighbors.end(); it ++)
		if ((*it)->node == node)
		{
			(*it)->node = newnode;
			(*it)->length = newlen;
		}
}


