/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "split.h"

Split::Split()
				: BoolVec()
{
	ntaxa = 0;
}

Split::Split(const BoolVec aBoolVec)
				: BoolVec(aBoolVec)
{
	ntaxa = aBoolVec.size();
}

Split::Split(int antaxa)
				: BoolVec()
{
	setNTaxa(antaxa);
}

Split::Split(const Split &sp)
				: BoolVec(sp)
{
	ntaxa = sp.ntaxa;
}

Split::Split(int antaxa, vector<int> taxa_list)
				: BoolVec()
{	
	setNTaxa(antaxa);
	vector<int>::iterator it;
	for (it = taxa_list.begin(); it != taxa_list.end(); it++)
	{
		int value = *it;
		(*this)[value] = true;		
	}

}

void Split::setNTaxa(int antaxa)
{
	ntaxa = antaxa;
	resize(ntaxa,0);
	for (iterator it = begin(); it != end(); it++)
		(*it) = false;
}

void Split::printSplit(ostream &out)
{
	for (int i=0; i < getNTaxa(); i++)
	{
		if ((*this)[i] == true)
			out << "1";
		else
			out << "0";
	}
}
Split::~Split()
{
// 	(*this).clear();
}

bool Split::isEmpty()
{
	for (iterator it = begin(); it != end(); it++)
		if (*it == true) return false;
	return true;
}

void Split::addTaxon(int tax_id)
{
	assert(tax_id >= 0 && tax_id < ntaxa);
	(*this)[tax_id] = true;
}

void Split::removeTaxon(int tax_id)
{
	assert(tax_id >= 0 && tax_id < ntaxa);
	(*this) = false;
}

bool Split::operator==(const Split &sp) const
{
	if (ntaxa != sp.ntaxa) return false;
	bool comp = true;
	//directly comparison
	for (const_iterator it = begin(), it2 = sp.begin(); it != end(); it++, it2++)
		if ((*it) != (*it2))
			comp = false;
	if (comp == true)
		return comp;
	
	//compare with the inverse
	for (const_iterator it = begin(), it2 = sp.begin(); it != end(); it++, it2++)
		if ( (*it) == (*it2) )
			return false;
	return true;
}

Split &Split::operator+=(Split &sp)
{
	assert(sp.ntaxa == ntaxa);
	iterator it1, it2;
	for (it1 = begin(), it2 = sp.begin(); it1 != end(); it1++, it2++) {
		(*it1) = (*it1) or (*it2);
	}
	return *this;
}

Split &Split::operator*=(Split &sp)
{
	assert(sp.ntaxa == ntaxa);
	iterator it1, it2;
	for (it1 = begin(), it2 = sp.begin(); it1 != end(); it1++, it2++) {
		(*it1) = (*it1) and (*it2);
	}
	return *this;	
}

Split &Split::operator= (const Split &sp)
{
// 	assert(ntaxa == sp.ntaxa);
	BoolVec::operator= (sp);
	ntaxa = sp.getNTaxa();
	return *this;
}





