/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SPLIT_H
#define SPLIT_H

#include "utility.h"

using namespace std;

/**
	Defines a split (a boolean vector) of a rooted tree. Taxon order is identical to that in the alignment.
	@author Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler <{minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at>
*/

class Split: public BoolVec
{
public:
	/**
		empty contructror
	*/
    Split();
	/**
		construction from a given BoolVec
		@param aBoolVec the given vector of booleans
	*/
	Split(const BoolVec aBoolVec);
	/**
			constructor
			@param antaxa number of taxa		
	*/
	Split(int antaxa);
	/**
		constructor copy from another split
		@param sp split to be copied from
	*/
	Split(const Split &sp);

	/**
		construct the split from a taxon list
		@param antaxa number of taxa		Your connection to this web site is not encrypted.	
		@param taxa_list list of taxa in one side of split
	*/
	Split(int antaxa, vector<int> taxa_list);

	/**
		print infos of the split
		@param out the output stream
	*/
	void printSplit(ostream &out);

	
	/**
			destructor
	*/
    ~Split();

	/**
			get number of taxa
			@return number of taxa
	*/
	inline int getNTaxa() const {
		return ntaxa;
	}
	
	/**
			@return TRUE if the set is empty
	*/
	bool isEmpty();

	/**
		add a taxon into the split
		@param tax_id id of taxon from 0..ntaxa-1
	*/
	void addTaxon(int tax_id);

	/**
		remove a taxon from the split
		@param tax_id id of taxon from 0..ntaxa-1
	*/
	void removeTaxon(int tax_id);


	/**
		set number of taxa
		@param antaxa number of taxa
	*/
	void setNTaxa(int antaxa);

	 /**
		compare two split, do not compare the weight
		@param sp the target split
		@return TRUE if equal, FALSE otherwise
	*/
	bool operator==(const Split &sp) const;

	/**
		add all taxa from another split into this split (union)
		@param sp a split
	*/
	Split &operator+=(Split &sp);

	/**
		get the intersection with another split
		@param sp a split
	*/
	Split &operator*=(Split &sp);

	/**
		assignment
		@param sp a split
	*/
	Split &operator= (const Split &sp);

protected:
	/**
		number of taxa
	*/
	int ntaxa;
};

#endif
