/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SPLIT_H
#define SPLIT_H

#include <vector>
#include <string>
#include "utility.h"

using namespace std;

/**
	Defines a split of a rooted tree.
	@author Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler <{minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at>
*/

/**
	each integer number contains sizeof(UINT) bytes, i.e. sizeof(UINT)*8 bits. Each bit represents for a taxon, the bit is 1 if the taxon is in split, 0 otherwise. ==> To describe a split for a tree of N leaves, we need at least floor(N/UINT_BITS) + 1 integer numbers.
*/
class Split : public vector<UINT>
{
public:
	/**
		empty constructor
	*/
	Split();

	/**
		constructor
		@param antaxa number of taxa			
	*/
	Split(int antaxa);


	/**
		constructor copy from another split
		@param sp split to be copied from
	*/
	Split(const Split &sp);

	/**
		set number of taxa
		@param antaxa number of taxa
	*/
	void setNTaxa(int antaxa);

	/**
		construct the split from a taxa list
		@param antaxa number of taxa			
		@param taxa_list list of taxa in one side of split
	*/
	Split(int antaxa, vector<int> taxa_list);

	/**
		print infos of this split
		@param out the output stream
	*/
	void printSplit(ostream &out);
	
	/**
		get number of taxa
		@RETURN number of taxa
	*/
	inline int getNTaxa() const {
    	return ntaxa;
    }
	/**
        @return TRUE if the set is empty
    */
    bool isEmpty();

	/**
		compare two split
		@param sp the target split
		@return TRUE if equal, FALSE otherwise
	*/
	bool operator==(const Split &sp) const;

	/**
		add a taxon into the split
		@param tax_id id of taxon from 0..ntaxa-1
	*/
	void addTaxon(const int tax_id);

	/**
		add all taxa from another split into this split (union)
		@param sp a split
	*/
	Split &operator+=(Split &sp);

	/**
		get the intersection with another split
		@param sp a split
	*/
	Split &operator*=(Split &sp);

	/**
		assignment
		@param sp a split
	*/
	Split &operator= (const Split &sp);


protected:
	/**
		number of taxa
	*/
	int ntaxa;
};

#endif
