/***************************************************************************
 *   Copyright (C) 2009 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SPLITSET_H
#define SPLITSET_H

#include "utility.h"
#include "split.h"
#include "pattern.h"
#include "patternset.h"

/**
	@author Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler <{minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at>
*/
class SplitSet	: public vector<Split*>
{
public:
    friend class Tree;

    SplitSet() : vector< Split* >(){} ;
	/**
		release the memory of all element splits, then resize to 0
	*/
	void removeAll();

	/**
		print split set
		@param out output stream
	*/
	void print(ostream& out);

	/**
		generate a set of (at most max) patterns on this splitset given the source (a set of patterns), the mutation type and a referenceSet. The generated pattern should not be in the referenceSet.
		@param sourceSet a set of source patterns
		@param mutType the mutation type
		@param referenceSet the reference set
		@param max maximum number of patterns should be generated
		(the number of pattern generated = min {max, number of possible patterns can be generated from sourceSet that not in referenceSet}
		@param returnSet (OUT) the generated pattern set
	*/
//  	void genPatterns(const PatternSet &sourceSet, PatternSet &returnSet, const int mutType, const PatternSet &referenceSet, const int max);

	/**
		generate a set of pattern on this splitset given the source (a set of pattern) and 2 reference set. The generated pattern will be return via returnSet if it's not be in neither referSet nor referSet2. If it is in referSet2, then add it to returnSet2. Patterns will be generated according to all 3 mutation type.
		@param sourceSet a set of source patterns
		@param referSet1 main reference set 
		@param referSet2 reference set 2
		@param returnSet (OUT) main return set
		@param returnSet2 (OUT) if the generated pattern not in referSet but in referSet2, it will be added into returnSet2
		@param aMutStep specify the step of mutations
	*/
// 	void genPatterns(const PatternSet &sourceSet, const PatternSet &referSet, const PatternSet &referSet2, PatternSet &returnSet, PatternSet &returnSet2, const int aMutStep);
	
	//without aMutStep, assign mutStep = mutStep of the source pattern + 1
	void genPatterns(const PatternSet &sourceSet, const PatternSet &referSet, const PatternSet &referSet2, PatternSet &returnSet, PatternSet &returnSet2);

	/**
		generate a set of (at most max) patterns on this splitset given the source (a set of patterns), the mutation type and a referenceSet. The generated pattern should not be in neither the referenceSet nor referenceSet2. If it is in referenceSet2, then add it to returnSet2.
		@param sourceSet a set of source patterns
		@param mutType the mutation type
		@param referenceSet the reference set
		@param referenceSet2 the second reference set
		@param max maximum number of patterns should be generated
		(the number of pattern generated = min {max, number of possible patterns can be generated from sourceSet that not in referenceSet}
		@param returnSet (OUT) the generated pattern set
		@param returnSet2 (OUT) if the generated pattern not in referenceSet but in referenceSet2, it will be add to this returnSet2.
	*/
//  	void genPatterns(const PatternSet &sourceSet, PatternSet &returnSet, const int mutType, const PatternSet &referSet, const int max, const PatternSet &referSet2, PatternSet &returnSet2);

	virtual ~SplitSet();
};

#endif
