/***************************************************************************
 *   Copyright (C) 2008 by Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler   *
 *   {minh.anh.nguyen, steffen.klaere, arndt.von.haeseler}@univie.ac.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "utility.h"

/*--------------------------------------------------------------*/
/*--------------------------------------------------------------*/

/**
	print error message then exit program
*/
void outError(const char *error){
	cerr << "ERROR: " << error << endl;
	exit(EXIT_FAILURE);
}

/**
	print error message then exit program
*/
void outError(string error){
	outError(error.c_str());
}

/*--------------------------------------------------------------*/
/*--------------------------------------------------------------*/

/**
	print double error messages then exit program
*/
void outError(const char *error, const char *msg){
	string str = error;
	str += msg;
	outError(str);
}

/**
	print double error messages then exit program
*/
void outError(const char *error, string msg){
	string str = error;
	str += msg;
	outError(str);
}

/*--------------------------------------------------------------*/
/*--------------------------------------------------------------*/

/**
	convert string to integer, with error checking
	@param str original string
	@return the integer value
*/
int convert_int(const char *str) throw (string){
	char *endptr;
	long i = strtol(str, &endptr, 10);

	if ((i == 0 && ((long) endptr - (const long) str) == 0) || abs(i) == HUGE_VALL || *endptr != 0) {
		string err = "Expecting integer, but found \"";
		err += str;
		err += "\" instead";
		throw err;
	}

	return i;
}

/**
	convert string to double, with error checking
	@param str original string
	@return the double
*/
double convert_double(const char *str) throw (string){
	char *endptr;
	double d = strtod(str, &endptr);
	if ((d == 0.0 && ((long) endptr - (const long) str) == 0) || fabs(d) == HUGE_VALF || *endptr != 0) {
		string err = "Expecting floating-point number, but found \"";
		err += str;
		err += "\" instead";
		throw err;
	}
	return d;
}

int searchString(string stringToFind, StringVec strVec)
{
        int vecSize;
        vecSize = strVec.size();
        for (int i=0; i<vecSize; i++){
			if (stringToFind == strVec[i])
			return i;
        }
        return -1;
}

double ordinaryRound(double num)
{
	/*double numFloor = floor(num);
	double numFmod = num - numFloor;
	return (numFmod < 0.5)? numFloor : (numFloor + 1);*/
	return floor(num + 0.5);
}

void printIntMatrix(ostream &out, IntMatrix matr)
{
	int row = matr.size();
	int col = matr[0].size();
	for (int i = 0; i < row; i++)
	{
		for (int j = 0; j < col; j++)
			out << matr[i][j] << " ";
		out << endl;
	}
}

void printBoolMatrix(ostream &out, BoolMatrix matr)
{
	int row = matr.size();
	int col = matr[0].size();
	out << row << " " << col << endl;
	for (int i = 0; i < row; i++)
	{
		for (int j = 0; j < col; j++)
			if (matr[i][j])
				out << "1 ";
			else out << "0 "; 
		out << endl;
	}
}

void initIntMatrix(IntMatrix &matr, int row, int col, int value)
{	
	IntVec aRow(col, value);
	IntMatrix _matr(row, aRow);
	matr = _matr;
	aRow.clear();
	_matr.clear();
}

void resetIntVector(IntVec &vec, int value)
{
	if (vec.empty())
		outError("Reset an empty vector in function resetIntVector!");
	int size = vec.size();
	IntVec _vec(size, value);
	vec = _vec;
	_vec.clear();
}

void resetBoolVector(BoolVec &vec, bool value)
{
	if (vec.empty())
		outError("Reset an empty vector in function resetBoolVector!");
	int size = vec.size();
	BoolVec _vec(size, value);
	vec = _vec;
	_vec.clear();
}

void resetIntMatrix(IntMatrix &matr, int value)
{
	if (matr.empty())
		outError("Reset an empty matrix in function resetIntMatrix!");
	int m = matr.size();
	for (int i = 0; i < m; i++)
	{
		resetIntVector(matr[i], value);
	}
}

IntVec getMutatedPattern(string pattern1, string pattern2)
{
	IntVec ret;
	int patLen = strlen( pattern1.c_str() );	
	for (int i = 0; i < patLen; i++)
	{
		int index1 = -1, index2 = -1;
		switch ( toupper(pattern1[i]) ){
			case 'A': index1 = 0; break;
			case 'C': index1 = 1; break;
			case 'G': index1 = 2; break;
			case 'T': index1 = 3; break;
			case 'U': index1 = 3; break;
			default: break;
		}
		switch ( toupper(pattern2[i]) ){
			case 'A': index2 = 0; break;
			case 'C': index2 = 1; break;
			case 'G': index2 = 2; break;
			case 'T': index2 = 3; break;
			case 'U': index1 = 3; break;
			default: break;
		}
		if (index1 == -1 || index2 == -1)
		{
			string errMsg = "There is non-{ACGT} in ";
			errMsg += pattern1;
			errMsg += " or ";
			errMsg += pattern2;
			outError(errMsg);
		}
		ret.push_back(K3stMatrix[index1][index2]);
	}
	return ret;
}

void printCopyright()
{
	cout << "\nmisfits: Evaluating the goodness of fit between a phylogenetic model and an alignment." << endl;
	cout << "\tVersion 1.0 - March 2010 release." << endl;
	cout << "Copyright (C) Minh Anh Thi Nguyen, Steffen Klaere, Arndt von Haeseler" << endl<<endl;
}

void printHelp(char* argv[])
{
	cout << "USAGE: " << argv[0] << " -a <file> -t <file> -pz <file> OPTIONS" << endl;
	cout << "REQUIRED arguments:" << endl;
	cout <<	"\t -a <file>:  file contains alignment in Phylip format." << endl;
	cout << "\t -t <file>:  file contains the tree in Newick format." << endl;		
	cout << "\t -pz <file>: file contains parameter's values to run TREE-PUZZLE." << endl;
	cout << "OPTIONS:" << endl;
	cout << "\t -h: print this help message." << endl;
	cout << "\t -s: to assign mutations on branches. Default is off." << endl;
	cout << "\t -p <file>: file contains paterns, frequencies, log_likelihood." << endl;
	cout << "\t\t Format: each line contains:" << endl; 
	cout << "\t\tpattern\tfrequency\tlog_likelihood" << endl;
	cout << "\t -pztree <file>: file contains the tree to run puzzle " << endl;
	cout << "\t\t if it is different from the general tree. " << endl;	
	cout << "\t -o <outgroupFile>: file contains outgroup of the tree." << endl;
	cout << "\t\t Format: each taxon name in one line." << endl;
	cout << "\t\t Default is the first taxon (leaf) in the tree file." << endl;	
	cout << "\t -noInit: to NOT include observed patterns which are in the confidence" << endl;
	cout << "\t\t region but not yet reach the upper bound into D- set (default is YES)." << endl;	
	//cout << "\t -stop   :   to stop if the program requires more than ~7.45GB of memory." << endl;
	cout << "\t -mdf <number>: degree of freedom (from the model used to construct the tree)" << endl;
	cout << "\t\t to compute confidence interval according to Chi-square dist. Default is 0." << endl;
	cout << "\t -ciBr: to include number of branches into degree of freedom to compute" << endl;
	cout << "\t\t confidence region according to Chi-square dist." << endl;
	cout << "\t\t When both -mdf and -ciBr are not set, confidence interval will be" << endl;
	cout << "\t\t computed according to normal distribution." << endl;
	cout << "\t -prefix <prefix>: set prefix for output files name. Default is alignment file name." << endl;
	cout << "ADDITIONAL INFORMATION:" << endl;
	cout << "\t -mstep <number>: to generate and print 1, 2 .. mstep mutation away patterns." << endl;
	cout << "\t\t Default is 0 (do not print)" << endl;
	cout << "\t -add: to print additional output files." << endl;
}
void parseArg(int argc, char* argv[], Params &params)
{
	//initialize
	params.alignFile = NULL;
	params.treeFile = NULL;
// 	params.patternFile = NULL;
// 	params.iqpnniPatternFile = NULL;
	params.patFile = NULL;
	params.outgroupFile = NULL;
	params.uprefix = NULL;
	params.ass = false;
	params.memStop =false;
	params.puzzleFile = NULL;
	params.ciBr = false;
	params.mdf = 0;
	params.noInit = false;
	params.pztree = NULL;
	params.mstep = 0;
	params.add = false;

	//set outfile
	params.outAll = "";
	params.outScore = "";
	params.outMatching = "";
	params.outMore = "";

	params.outMoreAlign="";
	params.outMatchingPat="";

	params.outLess = "";
	params.outScoreMatrix = "";
	params.outTreeBrLb = "";
	params.outTreeMin = "";
	params.outTreeMax = "";
	params.outTableMin = "";
	params.outTableMax = "";
	params.outOverPos = "";
	params.outAlignLess = "";
	params.outAlignPat = "";
	params.outAlignMoreLess = "";

	params.outgroup = StringVec();
	params.mutRunTime = 0;
	params.bmcRunTime = 0;
	params.assRunTime = 0;

	//parse argv
	int count;
	if (argc == 1)
	{
		printHelp(argv);
		exit(0);
	}
	try	{
		for (count = 1; count < argc; count++)
		{
			if ( strcmp(argv[count], "-h") == 0 || strcmp(argv[count], "--help") == 0)
			{
				printHelp(argv);
				exit(0);
			}
			else if ( strcmp(argv[count], "-a") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -a <alignment file name>";
				params.alignFile = argv[count];
			}			
			else if ( strcmp(argv[count], "-t") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -t <tree file name>";
				params.treeFile = argv[count];
			}
			else if ( strcmp(argv[count], "-pztree") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -pztree <tree file name>";
				params.pztree = argv[count];
			}
			else if ( strcmp(argv[count], "-prefix") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -prefix <prefix of output files' name>";
				params.uprefix = argv[count];
			}
			else if ( strcmp(argv[count], "-p") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -p <pattern with its frequence and log likelihood file name>";
				params.patFile = argv[count];
			}
			else if ( strcmp(argv[count], "-o") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -o <outgroupFile>";
				params.outgroupFile = argv[count];
			}
			else if ( strcmp(argv[count], "-pz") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -pz <params for TREE-PUZZLE>";
				params.puzzleFile = argv[count];
			}
			else if (strcmp(argv[count], "-s") == 0)
			{
				params.ass = true;
			}
			else if (strcmp(argv[count], "-stop") == 0)
			{
				params.memStop = true;
			}
			else if (strcmp(argv[count], "-ciBr") == 0)
			{
				params.ciBr = true;
			}
			else if (strcmp(argv[count], "-noInit") == 0)
			{
				params.noInit = true;
			}
			else if (strcmp(argv[count], "-add") == 0)
			{
				params.add = true;
			}
			else if ( strcmp(argv[count], "-mdf") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -mdf <number>: degree of freedom from the model";
				params.mdf = convert_int(argv[count]);
			}
			else if ( strcmp(argv[count], "-mstep") == 0) 
			{
				count++;
				if (count >= argc)
					throw "use -mstep <number>: to print patterns which are mstep away from constant patterns.";
				params.mstep = convert_int(argv[count]);
			}
			else
			{
				string err = "Unknown argument \"";
				err += argv[count];
				err += "\"";
				throw err;
			}
		}
	} catch (const char* str) {
		outError(str);
	} catch (char* str) {
		outError(str);
	} catch (string str) {
		outError (str);
	} catch (...) {
		string err = "Unknown argument \"";
		err += argv[count];
		err += "\"";
		outError(err);
	}

	//finish reading command line argument
	if (params.alignFile == NULL)
		outError("No alignment file is given. Please use -a!");
	/** //will be used latter
	if (params.treeFile == NULL && patternFile != NULL)
		outError("having pattern file but missing tree file!");
	if (params.treeFile == NULL && iqpnniPatternFile != NULL)
		outError("having pattern file from iqpnni program but missing tree file!");*/
	if (params.treeFile == NULL)
		outError("No tree file is given. Please use -t!");
	if (params.puzzleFile == NULL)
		outError("Missing file containing params to run TREE-PUZZLE. Please use -pz!");
	if (params.pztree == NULL)
		params.pztree = params.treeFile;
	if (params.outgroupFile != NULL)
		readStringVec(params.outgroupFile, params.outgroup);
	if (!params.outgroup.empty())
	{
		cout << "The outgroup taxa are: " << endl;
		for (StringVec::iterator it = params.outgroup.begin(); it != params.outgroup.end(); it++)
			cout << (*it) << endl;
	}
	if (params.uprefix == NULL)
		params.uprefix = params.alignFile;
// 	if (params.patFile == NULL)
// 		outError("Missing file containing pattern, pattern's frequency and loglikelihood. Please use -p!");		
}

/*void readPatllFile (const char* inFile, int &seqLen, int &pattNum, StringVec &patterns, IntVec &observedNum, IntVec &expectedNum)
{
	cout << "Reading pattern containing file: " << inFile << "...." << endl;
	
	ifstream in;
	try{
		in.exceptions (ios::failbit | ios::badbit);
		in.open(inFile);
 		readPatllStream(in, seqLen, pattNum, patterns, observedNum, expectedNum);
		in.clear();		
		in.exceptions (ios::failbit | ios::badbit);
		in.close();
  	} catch (ios::failure){
		outError(ERR_READ_INPUT, inFile);
	}
	
	cout << "Finish reading file!" << endl;
	cout << "File contains " << pattNum << " gap-free patterns. Gap-free sequence's length is " << seqLen << endl;
} 

void readPatllStream (ifstream &inFile, int &seqLen, int &pattNum, StringVec &patterns, IntVec &observedNum, IntVec &expectedNum)
{
	int _pattNum = 0; //number of all patterns
	int _seqLen = 0; //sequence's length without gaps or unknown character
	StringVec _patterns;
	IntVec _observedNum;
	IntVec _expectedNum;

	DoubleVec LogLL;

	string currentPattern, currentObservedStr, currentLogLLStr, currentProStr;
	int currentObserved;
	double currentLogLL;
	double currentPro;

	try{					
		while (!inFile.eof())
		{	
			//reading each line of the file
			//remove the badbit
  			inFile.exceptions (ios::badbit);	
			if ( !(inFile >> currentPattern >> currentObservedStr >> currentLogLLStr >> currentProStr) ) break;
			//set the failbit again			
 			inFile.exceptions (ios::failbit | ios::badbit);			
			
			//if the pattern is gap-free, take it into account
			if (strcspn(currentPattern.c_str(), UGAP) == strlen(currentPattern.c_str()))
			{
// 				cout << "Test in readPatllStream " << currentPattern << "\t" << currentObservedStr << "\t" << currentLogLLStr << "\t" << currentProStr << endl;

				currentObserved = convert_int(currentObservedStr.c_str());
				currentLogLL = convert_double(currentLogLLStr.c_str());
				currentPro = convert_double(currentProStr.c_str());
			
				_patterns.push_back(currentPattern);
				_pattNum++; 
				_seqLen += currentObserved;
				_observedNum.push_back(currentObserved);
				LogLL.push_back(currentLogLL);				
			}
		}					
	} catch(bad_alloc){
		outError(ERR_NO_MEMORY);
	} catch (const char *str){
		outError(str);
	} catch (char *str){
		outError(str);
	} catch (string str){
		outError(str);
  	} catch (ios::failure){
  		outError(ERR_READ_INPUT);
 	} catch (...){
 		outError(ERR_READ_ANY);
	}
	
	//compute the expected number of frequency of each pattern given its loglikelihood
	computeExOccur(_seqLen, _pattNum, LogLL, _expectedNum);
	
	//return value
	seqLen 		= _seqLen;
	pattNum 	= _pattNum;
	patterns 	= _patterns;
	observedNum	= _observedNum;
	expectedNum = _expectedNum;	
}*/

/*void computeExOccur(int seqLen, int pattNum, DoubleVec logLL, IntVec &expectedNum)
{
	//compute expected number of occurence from log-likelihood and scale it by adding logarithm of 4^seqence's length, we use log-likelihood since the probability (outputed by iqpnni) may be very small
	IntVec _expectedNum;
 	double scaleFactor = log(4^seqLen);
	DoubleVec prob;
	DoubleVec partLen;
	double sumPartLen = 0;
	double tempProb;
	for (int i = 0; i < pattNum; i++)
	{
		tempProb = exp(logLL[i] + scaleFactor);
		prob.push_back(tempProb);
		sumPartLen += tempProb;		
	}
	for (int i = 0; i < pattNum; i++)
	{
		partLen.push_back(prob[i] * seqLen / sumPartLen);
	}

	//the first expected number of occurence
	_expectedNum.push_back((int)ordinaryRound(partLen[0]));
	// update partLen from the second element, then update _expectedNum	
	for (int i = 1; i < pattNum; i++)
	{
		partLen[i] += (partLen[i-1] - ordinaryRound(partLen[i-1]));
		_expectedNum.push_back((int)ordinaryRound(partLen[i]));
	}
	expectedNum = _expectedNum;	
}*/
void readStringVec(const char* inFile, StringVec &retVec)
{
	cout << "Reading a set of strings, e.g. taxa names, in " << inFile << "..." <<endl;
	ifstream in;
	try{
		in.exceptions (ios::failbit | ios::badbit);
		in.open(inFile);
		readStringVec(in, retVec);
		in.clear();
		in.exceptions (ios::failbit | ios::badbit);
		in.close();
	} catch (ios::failure){
		outError(ERR_READ_INPUT, inFile);
	}
	cout << "Finish reading!" << endl;
}

void readStringVec(ifstream &inFile, StringVec &retVec)
{
	StringVec _retVec;
	string currentString;	
	try{				
		while (!inFile.eof())
		{
			//reading each line of the file
			//remove the badbit
			inFile.exceptions (ios::badbit);
			if ( !(inFile >> currentString) ) break;
			//set the failbit again
			inFile.exceptions (ios::failbit | ios::badbit);
			_retVec.push_back(currentString);
		}
	} catch(bad_alloc){
			outError(ERR_NO_MEMORY);
	} catch (const char *str){
			outError(str);
	} catch (char *str){
			outError(str);
	} catch (string str){
			outError(str);
	} catch (ios::failure){
			outError(ERR_READ_INPUT);
	} catch (...){
			outError(ERR_READ_ANY);
	}
	retVec = _retVec;
}
/*20100310
void computeExOccur(int seqLen, int pattNum, DoubleVec logLL, IntVec &expectedNum)
{
	DoubleVec sortedLogLL = logLL;
	sort(sortedLogLL.begin(), sortedLogLL.end());
*/
	//compute expected number of occurence from log-likelihood and scale it by adding logarithm of 4^seqence's length, we use log-likelihood since the probability (outputed by iqpnni) may be very small
	/*20100310
	IntVec _expectedNum;
 	double scaleFactor = -sortedLogLL[pattNum-1];
	LongDoubleVec prob;
	LongDoubleVec partLen;
	long double sumPartLen = 0;
	long double tempProb;
	for (int i = 0; i < pattNum; i++)
	{
		tempProb = expl(logLL[i] + scaleFactor);
		prob.push_back(tempProb);
//  	sumPartLen += tempProb;		
	}
	*/

	//calculate sum of all probabilities after sorting them
	/* 20100310
	LongDoubleVec sortedProb = prob;
	sort(sortedProb.begin(), sortedProb.end());
	for (int i = 0; i < pattNum; i++)
	{
		sumPartLen += sortedProb[i];
	}
	*/

	/*bool stop = false;
	cout << "sumPartLen: " << sumPartLen << endl;
	long double testSumPartLen = sumPartLen;
	for (int i = pattNum-1; i >= 0 && !stop; i--)
	{
		cout << "index: " << i << "\t testSumPartLen: " << testSumPartLen << endl;
		testSumPartLen -= sortedProb[i];
		if (testSumPartLen == 0)
		{
			cout << "sum stop at: " << i << endl;
			stop = true;
		}
	}

	if (stop == true)
		cout << "stop is true" << endl;	*/	
	
// 	cout << "sortedLogLL and sortedProb: " << endl;
	//compute each partLen
	/* 20100310
	for (int i = 0; i < pattNum; i++)
	{
		partLen.push_back(prob[i] * seqLen / sumPartLen);
	}
	*/
// 	cout << endl;
	//compute sum of all partLen to check whether it's close to seqLen
/*	long double checkSumPartLen = 0;
	for (int i=0; i < pattNum; i++)
		checkSumPartLen += partLen[i];
 	cout << "\ncheckSumPartLen: " << checkSumPartLen << ", gap-free sequence length: " << seqLen << endl;*/
	
	//the first expected number of occurence
	/* 20100310
	_expectedNum.push_back((int)ordinaryRound(partLen[0]));
	*/
	// update partLen from the second element, then update _expectedNum	
	/* 20100310
	for (int i = 1; i < pattNum; i++)
	{
		partLen[i] += (partLen[i-1] - ordinaryRound(partLen[i-1]));
		_expectedNum.push_back((int)ordinaryRound(partLen[i]));
	}
	expectedNum = _expectedNum;
	
}
*/


/*void computeConInter(int seqLen, DoubleVec logLLs, DoubleVec &upperBounds, DoubleVec &lowerBounds)
{
	cout << "Computing Wald confidence interval ..." << endl;
	double kappa = QnormTable[seqLen-1];	
	int patNum = logLLs.size();	
	DoubleVec _upperBounds, _lowerBounds;
	double upperB, lowerB, ll, diff;
	for (int i = 0; i < patNum; i++)
	{
		ll = exp(logLLs[i]);
		diff = kappa*sqrt(ll*(1-ll)/seqLen);
		upperB = ll + diff;
		if (ll<diff)
			lowerB = 0;
		else 
			lowerB = ll-diff;
		_upperBounds.push_back(upperB);
		_lowerBounds.push_back(lowerB);	
	}
	//return values
	upperBounds = _upperBounds;
	lowerBounds = _lowerBounds;
	_upperBounds.clear();
	_lowerBounds.clear();
	cout << "Finish computing Wald confidence interval." << endl;
}*/

void removeSpaces(string &str)
{
	while(str.find(" ") != string::npos)
  	{
    	str.replace(str.find(" "), 1, "");
  	}	
}

/*void computeDiff (StringVec patternVec, int pattNum, IntVec observedNum, IntVec expectedNum, StringVec &morePatterns, StringVec &lessPatterns, IntVec &moreAmounts, IntVec &lessAmounts)
{
	int currentDiff;
	StringVec _morePatterns, _lessPatterns;
	IntVec _moreAmounts, _lessAmounts;
	for (int i = 0; i < pattNum; i++)
	{
		currentDiff = observedNum[i] - expectedNum[i];
		//observed less than expected
		if (currentDiff < 0)
		{
			_lessPatterns.push_back(patternVec[i]);
			_lessAmounts.push_back(-currentDiff);
		}
		//observed more than expected
		if (currentDiff > 0)
		{
			_morePatterns.push_back(patternVec[i]);
			_moreAmounts.push_back(currentDiff);
		}
	}
	//return values
	lessPatterns = _lessPatterns;
	lessAmounts = _lessAmounts;
	morePatterns = _morePatterns;
	moreAmounts = _moreAmounts;
}*/

/*void computeExOccur(int seqLen, int pattNum, DoubleVec logLL, IntVec &expectedNum)
{
	//compute expected number of occurence from log-likelihood and scale it by adding logarithm of 4^seqence's length, we use log-likelihood since the probability (outputed by iqpnni) may be very small
	IntVec _expectedNum;
 	double scaleFactor = seqLen*log(4);
	DoubleVec prob;
	DoubleVec partLen;
	double sumPartLen = 0;
	double tempProb;
	for (int i = 0; i < pattNum; i++)
	{
		tempProb = exp(logLL[i] + scaleFactor);
		prob.push_back(tempProb);
		sumPartLen += tempProb;		
	}
	for (int i = 0; i < pattNum; i++)
	{
		partLen.push_back(prob[i] * seqLen / sumPartLen);
	}

	//the first expected number of occurence
	_expectedNum.push_back((int)ordinaryRound(partLen[0]));
	// update partLen from the second element, then update _expectedNum	
	for (int i = 1; i < pattNum; i++)
	{
		partLen[i] += (partLen[i-1] - ordinaryRound(partLen[i-1]));
		_expectedNum.push_back((int)ordinaryRound(partLen[i]));
	}
	expectedNum = _expectedNum;	
}*/

/*void readSeqNameFile (const char* inFile, int &seqNum, StringVec &seqNames)
{
	cout << "Reading sequence's names in " << inFile << "..." <<endl;
	ifstream in;
	try{
		in.exceptions (ios::failbit | ios::badbit);
		in.open(inFile);
		readSeqNameStream(in, seqNum, seqNames);
		in.clear();
		in.exceptions (ios::failbit | ios::badbit);
		in.close();
	} catch (ios::failure){
		outError(ERR_READ_INPUT, inFile);
	}
	cout << "Finish reading sequence's names!" << endl;
}*/


/*void readSeqNameStream (ifstream &in, int &seqNum, StringVec &seqNames)
{
	int _seqNum;
	StringVec _seqNames;
	int seqLen;
	string currentName, seqNumStr, seqLenStr;
	try {
		//get number of sequence in string
		in >> seqNumStr;
		//get sequence's length in string
		in >> seqLenStr;
		// convert to int
		_seqNum = convert_int(seqNumStr.c_str());
		seqLen = convert_int(seqLenStr.c_str());
		
		int ignoreMax = seqLen + MAX_NUM_SPACE;

		for (int i = 0; i < _seqNum; i++)
		{
			//get the name of sequence
			if ( !(in >> currentName) ) break;
			_seqNames.push_back(currentName);
			in.ignore(ignoreMax,'\n');
		}		
	} catch (bad_alloc) {
		outError(ERR_NO_MEMORY);
	} catch (const char* str) {
		outError(str);
	} catch (char *str) {
		outError(str);
	} catch (string str) {
		outError(str);
	} catch (ios::failure) {
		outError(ERR_READ_INPUT);
	} catch (...) {
		outError(ERR_READ_ANY);
	}
	//return
	seqNum = _seqNum;
	seqNames = _seqNames;
}*/

int getMin(IntVec inputVec, int indexSet)
{
	int vecLen = inputVec.size();
	if (inputVec.empty())	
		outError("getMin of an empty integer vector!");
	if (indexSet == 1) return inputVec[0];
	int min = MaxExBrCost;
	if (indexSet == 0)
	{
		for (int i = 0; i < vecLen; i++)
		{
			if (min > inputVec[i])
				min = inputVec[i];
		}
	}
	else
		for (int i = 0; i < vecLen; i++)
			if ((indexSet & MutState[i]) != 0 && min > inputVec[i])
				min = inputVec[i];
	return min;
}

int getMax(IntVec inputVec, int indexSet)
{
	int vecLen = inputVec.size();
	if (inputVec.empty())	
		outError("getMin of an empty integer vector!");
	if (indexSet == 1) return inputVec[0];
	int max = -2;
	if (indexSet == 0)
	{
		for (int i = 0; i < vecLen; i++)
		{
			if (max < inputVec[i])
				max = inputVec[i];
		}string intVec2Str(IntVec avec);
	}
	else
		for (int i = 0; i < vecLen; i++)
			if ((indexSet & MutState[i]) != 0 && max < inputVec[i])
				max = inputVec[i];
	return max;
}

int getRandomIndex(IntVec inputVec, int indexSet, bool min)
{	
	if (indexSet == 1) return 0;

	int minMax;

	if (min == true)
		minMax = getMin (inputVec, indexSet);		
	else
		minMax = getMax (inputVec, indexSet);

	int vecLen = inputVec.size();
	int count = 0;
	IntVec possibleIndex;
	if (indexSet == 0)
	{
		for (int i = 0; i < vecLen; i++)
			if (inputVec[i] == minMax)
			{
				count++;
				possibleIndex.push_back(i);
			}
	}
	else
	{
		for (int i = 0; i < vecLen; i++)
		if ((indexSet & MutState[i]) != 0 && inputVec[i] == minMax)
		{
			count++;
			possibleIndex.push_back(i);
		}
	}
	
	//only one minMax
	if (count == 1)
	{		
		return possibleIndex[0];
	}
	else
	{
		int ran;
		ran = genRanInt(count);
		return possibleIndex[ran];
	}
}

int genRanInt(const int max){
	return rand()%max;
}

void readSitelh (const char* inFile, DoubleVec &logllVec)
{
	cout << "\nReading file containing site's loglikelihood: " << inFile << "...." << endl;
    ifstream in;
	try{
		in.exceptions (ios::failbit | ios::badbit);
		in.open(inFile);
		readSitelh(in, logllVec);
		in.clear();
		in.exceptions (ios::failbit | ios::badbit);
		in.close();
	} catch (ios::failure){
		outError(ERR_READ_INPUT, inFile);
	}
	cout << "Finish reading site's loglikelihood!" << endl;	
}

void readSitelh (ifstream &inFile, DoubleVec &logllVec)
{
	DoubleVec _logllVec;
	int siteNum;
	string currentString;
	double currentLogLL;	
	try{
		//ignore the first number
		inFile >> currentString;
		//read number of sites
		inFile >> currentString;
		siteNum = convert_int(currentString.c_str());
		//ignore the name of the tree		
		inFile >> currentString;		
		while (!inFile.eof())
		{
			//reading each line of the file
			//remove the badbit
			inFile.exceptions (ios::badbit);
			if ( !(inFile >> currentString) ) break;
			//set the failbit again
			inFile.exceptions (ios::failbit | ios::badbit);
			_logllVec.push_back(convert_double(currentString.c_str()));
		}
	} catch(bad_alloc){
			outError(ERR_NO_MEMORY);
	} catch (const char *str){
			outError(str);
	} catch (char *str){
			outError(str);
	} catch (string str){
			outError(str);
	} catch (ios::failure){
			outError(ERR_READ_INPUT);
	} catch (...){
			outError(ERR_READ_ANY);
	}
	if (siteNum != _logllVec.size())
		outError("Actual number of site's likelihoods is not consistent with the announced number in the first line.");
	logllVec = _logllVec;
}

IntMatrix decomposeInterval(const IntVec intVec)
{
	IntMatrix ret;
	int prev = -2;
	int index = -1;
	int current;
	for ( IntVec::const_iterator it = intVec.begin(); it != intVec.end(); it++)
	{
		current = (*it);
		if (current - prev > 1)
		{
			index++;
			IntVec temp;
			temp.push_back(current);
			temp.push_back(1);
			ret.push_back(temp);
		}
		else
		{
			ret[index][1]++;
		}
		prev = current;
	}
	return ret;
}

string mutatedVec2Str(IntVec intVec)
{
	string ret;
	for ( IntVec::const_iterator it = intVec.begin(); it != intVec.end(); it++)
	{
		switch ( (*it) ){
			case 1: ret.push_back('0'); break;
			case 2: ret.push_back('1'); break;
			case 4: ret.push_back('2'); break;
			case 8: ret.push_back('3'); break;			
			default: break;
		}
	}
	return ret;
}
